# Generated by Django 5.0.12 on 2025-09-08 11:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wbfdm', '0032_alter_instrumentprice_outstanding_shares'),
    ]

    operations = [
        migrations.AlterField(
            model_name='controversy',
            name='review',
            field=models.DateField(blank=True, null=True, verbose_name='Reviewed'),
        ),
    ]
