# Generated by Django 5.0.8 on 2024-08-09 14:20

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("currency", "0001_initial"),
        ("geography", "0001_initial"),
        ("io", "0007_alter_exportsource_query_params"),
        ("tags", "0001_initial"),
        ("wbfdm", "0024_option_open_interest_10d_option_volume_10d_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="instrument",
            name="is_primary",
            field=models.BooleanField(blank=True, null=True),
        ),
        migrations.AddConstraint(
            model_name="instrument",
            constraint=models.UniqueConstraint(
                condition=models.Q(("is_primary", True)),
                fields=("parent", "is_primary"),
                name="unique_instrument_primary",
            ),
        ),
    ]
