# Generated by Django 5.0.6 on 2024-07-08 11:12

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0023_instrument_unique_ric_instrument_unique_rmc_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="option",
            name="open_interest_10d",
            field=models.FloatField(
                blank=True, help_text="Option Open Interest (10D)", null=True, verbose_name="Open Interest 10D"
            ),
        ),
        migrations.AddField(
            model_name="option",
            name="volume_10d",
            field=models.FloatField(blank=True, help_text="Option Volume (10D)", null=True, verbose_name="Volume 10D"),
        ),
        migrations.AddField(
            model_name="optionaggregate",
            name="open_interest_10d",
            field=models.FloatField(
                blank=True, help_text="Option Open Interest (10D)", null=True, verbose_name="Open Interest 10D"
            ),
        ),
        migrations.AddField(
            model_name="optionaggregate",
            name="volume_10d",
            field=models.FloatField(blank=True, help_text="Option Volume (10D)", null=True, verbose_name="Volume 10D"),
        ),
    ]
