# Generated by Django 4.1.9 on 2023-07-19 07:42

import django.db.models.deletion
import wbfdm.models.fk_fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0004_rename_financials_instrumentlookup_statements_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="instrument",
            name="corporate_action_backend",
            field=wbfdm.models.fk_fields.CompositeForeignKey(
                null=True,
                null_if_equal=[],
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="wbfdm.instrumentlookup",
                to_fields={
                    "corporate_actions": wbfdm.models.fk_fields.RawFieldValue(True),
                    "instrument_id": wbfdm.models.fk_fields.LocalFieldValue("id"),
                },
            ),
        ),
    ]
