# Generated by Django 4.2.9 on 2024-03-29 14:17

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0014_alter_controversy_instrument"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="instrument",
            index=models.Index(fields=["is_investable_universe"], name="instrument_investible_index"),
        ),
    ]
