# Generated by Django 5.0.12 on 2025-11-24 15:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wbfdm', '0033_alter_controversy_review'),
    ]

    operations = [
        migrations.AlterField(
            model_name='instrumentlist',
            name='instrument_list_type',
            field=models.CharField(blank=True, choices=[('WATCH', 'Watch List'), ('EXCLUSION', 'Exclusion List'), ('INCLUSION', 'Inclusion List')], max_length=32, null=True),
        ),
    ]
