# Generated by Django 5.2.9 on 2025-12-09 08:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('currency', '0001_initial'),
        ('geography', '0002_geography_geography_geography_tree_i739a'),
        ('io', '0008_importsource_resource_kwargs'),
        ('tags', '0001_initial'),
        ('wbfdm', '0034_alter_instrumentlist_instrument_list_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='instrument',
            name='related_instruments',
            field=models.ManyToManyField(blank=True, related_name='benchmarks_of', through='wbfdm.RelatedInstrumentThroughModel', through_fields=('instrument', 'related_instrument'), to='wbfdm.instrument', verbose_name='The Related Instruments'),
        ),
        migrations.AlterField(
            model_name='instrumentclassificationthroughmodel',
            name='related_instruments',
            field=models.ManyToManyField(blank=True, limit_choices_to=models.Q(('instrument_type__is_classifiable', True), ('level', 0)), through='wbfdm.InstrumentClassificationRelatedInstrument', through_fields=('classified_instrument', 'related_instrument'), to='wbfdm.instrument'),
        ),
        migrations.AddIndex(
            model_name='instrument',
            index=models.Index(fields=['tree_id', 'lft'], name='wbfdm_instrument_tree_id_lf91c'),
        ),
    ]
