# Generated by Django 5.2.9 on 2025-12-16 15:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wbfdm', '0035_alter_instrument_related_instruments_and_more'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='relatedinstrumentthroughmodel',
            unique_together=set(),
        ),

    ]
